@@ -1,16 +1,17 @@
 # Copyright Notice:
 # Copyright 2016-2021 DMTF. All rights reserved.
 # License: BSD 3-Clause License. For full text see link:
-# https://github.com/DMTF/python-redfish-library/blob/master/LICENSE.md
+# https://github.com/DMTF/python-redfish-library/blob/main/LICENSE.md
 
 """ Redfish restful library """
 
-__all__ = ['rest', 'ris', 'discovery']
-__version__ = "3.1.0"
+__all__ = ['rest', 'ris', 'discovery', 'messages']
+__version__ = "3.3.2"
 
 from redfish.rest.v1 import redfish_client
 from redfish.rest.v1 import AuthMethod
 from redfish.discovery.discovery import discover_ssdp
+from redfish.messages import *
 import logging
 
 def redfish_logger(file_name, log_format, log_level=logging.ERROR):
