@@ -1,7 +1,7 @@
 # Copyright Notice:
 # Copyright 2016-2021 DMTF. All rights reserved.
 # License: BSD 3-Clause License. For full text see link:
-# https://github.com/DMTF/python-redfish-library/blob/master/LICENSE.md
+# https://github.com/DMTF/python-redfish-library/blob/main/LICENSE.md
 
 # -*- coding: utf-8 -*-
 """Helper module for working with REST technology."""
@@ -15,7 +15,10 @@ import json
 import base64
 import logging
 import warnings
+import re
 import requests
+import requests_unixsocket
+from redfish.messages import *
 
 from collections import (OrderedDict)
 
@@ -44,9 +47,15 @@ class InvalidCredentialsError(Exception):
     """Raised when invalid credentials have been provided."""
     pass
 
+class SessionCreationError(Exception):
+    """Raised when a session could not be created."""
+    pass
+
 class ServerDownOrUnreachableError(Exception):
     """Raised when server is unreachable."""
-    pass
+    def __init__(self,message,*,response=None):
+        super().__init__(message)
+        self.response = response
 
 class DecompressResponseError(Exception):
     """Raised when decompressing response failed."""
@@ -164,14 +173,16 @@ class RestResponse(object):
         self._http_response = http_response
 
         if http_response is not None:
-            self._read = http_response.text
+            self._read = http_response.content
             self._status = http_response.status_code
 
     @property
     def read(self):
-        """Wrapper around httpresponse.read()"""
+        """Property for accessing raw content as an array of bytes (unless overridden)
 
-        # Backwards compatibility: for requests, we can simply use "text"; need to see if there are external dependencies to continue using read
+        TODO: Need to review usage elsewhwere; by default _read is an array of bytes, but applying a new value with a
+        setter routine will make it a string.  We might want to consider deprecating the setters.
+        """
         return self._read
 
     @read.setter
@@ -218,7 +229,11 @@ class RestResponse(object):
     @property
     def text(self):
         """Property for accessing the data as an unparsed string"""
-        return self.read
+        if isinstance(self.read, str):
+            value = self.read
+        else:
+            value = self.read.decode("utf-8", "ignore")
+        return value
 
     @text.setter
     def text(self, value):
@@ -234,10 +249,18 @@ class RestResponse(object):
     def dict(self):
         """Property for accessing the data as an dict"""
         try:
-            return json.loads(self.read)
+            if len(self.text) == 0:
+                # No response body; return empty dict instead to avoid exceptions
+                # No response bodies can be valid in many cases (especially 4XX and 5XX responses)
+                return {}
+            return json.loads(self.text)
         except:
+            if self.status == 500:
+                # Make an allowance for 500 status codes
+                # Depending on the reason for the error, it's possible the web server may not be able to support Redfish handling
+                return {}
             str = "Service responded with invalid JSON at URI {}\n{}".format(
-                self._rest_request.path, self.read)
+                self._rest_request.path, self.text)
             LOGGER.error(str)
             raise JsonDecodingError(str) from None
 
@@ -265,8 +288,14 @@ class RestResponse(object):
         """Property for accessing the saved session location"""
         if self._session_location:
             return self._session_location
+        if self.status not in [200, 201, 202, 204]:
+            return None
 
         self._session_location = self.getheader('location')
+        if self._session_location is None:
+            warnings.warn("Service incorrectly did not provide the 'Location' response header when creating a session; attempting to "
+                          "get the session location from the response body.  Contact your vendor.")
+            self._session_location = self.dict["@odata.id"]
         return self._session_location
 
     @property
@@ -402,10 +431,32 @@ class StaticRestResponse(RestResponse):
                 returnlist.append((key, value))
         else:
             for item in self._headers:
-                returnlist.append(item.items()[0])
+                returnlist.append(item)
 
         return returnlist
 
+    def getheader(self, name):
+        """Property for accessing an individual header
+
+        :param name: The header name to retrieve.
+        :type name: str.
+        :returns: returns a header from HTTP response
+        """
+        returnheader = None
+
+        if isinstance(self._headers, dict):
+            for key, value in self._headers.items():
+                if key.lower() == name.lower():
+                    returnheader = self._headers[key]
+                    break
+        else:
+            for item in self._headers:
+                if item[0].lower() == name.lower():
+                    returnheader = item[1]
+                    break
+
+        return returnheader
+
 class AuthMethod(object):
     """AUTH Method class"""
     BASIC = 'basic'
@@ -417,7 +468,7 @@ class RestClientBase(object):
     def __init__(self, base_url, username=None, password=None,
                                 default_prefix='/redfish/v1/', sessionkey=None,
                                 capath=None, cafile=None, timeout=None,
-                                max_retry=None):
+                                max_retry=None, proxies=None, check_connectivity=True):
         """Initialization of the base class RestClientBase
 
         :param base_url: The URL of the remote system
@@ -438,6 +489,11 @@ class RestClientBase(object):
         :type timeout: int
         :param max_retry: Number of times a request will retry after a timeout
         :type max_retry: int
+        :param proxies: Dictionary containing protocol to proxy URL mappings
+        :type proxies: dict
+        :param check_connectivity: A boolean to determine whether the client immediately checks for
+        connectivity to the base_url or not.
+        :type check_connectivity: bool
 
         """
 
@@ -447,14 +503,20 @@ class RestClientBase(object):
         self.__session_key = sessionkey
         self.__authorization_key = None
         self.__session_location = None
-        self._session = requests.Session()
+        if self.__base_url.startswith('http+unix://'):
+            self._session = requests_unixsocket.Session()
+        else:
+            self._session = requests.Session()
         self._timeout = timeout
         self._max_retry = max_retry if max_retry is not None else 10
+        self._proxies = proxies
         self.login_url = None
         self.default_prefix = default_prefix
         self.capath = capath
         self.cafile = cafile
-        self.get_root_object()
+
+        if check_connectivity:
+            self.get_root_object()
 
     def __enter__(self):
         self.login()
@@ -548,9 +610,19 @@ class RestClientBase(object):
         except Exception as excp:
             raise excp
 
+        if resp.status == 401 and self.__authorization_key is None and self.__session_key is None:
+            # Workaround where the service incorrectly rejects access to service
+            # root when no credentials are provided
+            warnings.warn("Service incorrectly responded with HTTP 401 Unauthorized for the service root.  Contact your vendor.")
+            self.root = {}
+            self.root_resp = resp
+            return
+        if resp.status == 503:
+            raise ServerDownOrUnreachableError("Service is busy, " \
+                                               "return code: %d" % resp.status,response=resp)
         if resp.status != 200:
-            raise ServerDownOrUnreachableError("Server not reachable, " \
-                                               "return code: %d" % resp.status)
+            raise ServerDownOrUnreachableError("Service not able to provide the service root, " \
+                                               "return code: %d" % resp.status,response=resp)
 
         content = resp.text
 
@@ -565,106 +637,130 @@ class RestClientBase(object):
         self.root = RisObject.parse(root_data)
         self.root_resp = resp
 
-    def get(self, path, args=None, headers=None):
+    def get(self, path, args=None, headers=None, timeout=None, max_retry=None):
         """Perform a GET request
 
-        :param path: the URL path.
-        :type path: str.
-        :param args: the arguments to get.
-        :type args: dict.
-        :param headers: dict of headers to be appended.
-        :type headers: dict.
+        :param path: The URI to access
+        :type path: str
+        :param args: The query parameters to provide with the request
+        :type args: dict, optional
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int, optional
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int, optional
         :returns: returns a rest request with method 'Get'
 
         """
         try:
             return self._rest_request(path, method='GET', args=args,
-                                                                headers=headers)
+                                                                headers=headers, timeout=timeout, max_retry=max_retry)
         except ValueError:
             str = "Service responded with invalid JSON at URI {}".format(path)
             LOGGER.error(str)
             raise JsonDecodingError(str) from None
 
-    def head(self, path, args=None, headers=None):
+    def head(self, path, args=None, headers=None, timeout=None, max_retry=None):
         """Perform a HEAD request
 
-        :param path: the URL path.
-        :type path: str.
-        :param args: the arguments to get.
-        :type args: dict.
-        :param headers: dict of headers to be appended.
-        :type headers: dict.
+        :param path: The URI to access
+        :type path: str
+        :param args: The query parameters to provide with the request
+        :type args: dict, optional
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int, optional
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int, optional
         :returns: returns a rest request with method 'Head'
 
         """
         return self._rest_request(path, method='HEAD', args=args,
-                                                                headers=headers)
+                                                                headers=headers, timeout=timeout, max_retry=max_retry)
 
-    def post(self, path, args=None, body=None, headers=None):
+    def post(self, path, args=None, body=None, headers=None, timeout=None, max_retry=None):
         """Perform a POST request
 
-        :param path: the URL path.
-        :type path: str.
-        :param args: the arguments to post.
-        :type args: dict.
-        :param body: the body to the sent.
-        :type body: str.
-        :param headers: dict of headers to be appended.
-        :type headers: dict.
+        :param path: The URI to access
+        :type path: str
+        :param args: The query parameters to provide with the request
+        :type args: dict, optional
+        :param body: The request body to provide; use a dict for a JSON body, list for multipart forms, bytes for an octet stream, or str for an unstructured request
+        :type body: dict or list or bytes or str, optional
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int, optional
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int, optional
         :returns: returns a rest request with method 'Post'
 
         """
         return self._rest_request(path, method='POST', args=args, body=body,
-                                                                headers=headers)
+                                                                headers=headers, timeout=timeout, max_retry=max_retry)
 
-    def put(self, path, args=None, body=None, headers=None):
+    def put(self, path, args=None, body=None, headers=None, timeout=None, max_retry=None):
         """Perform a PUT request
 
-        :param path: the URL path.
-        :type path: str.
-        :param args: the arguments to put.
-        :type args: dict.
-        :param body: the body to the sent.
-        :type body: str.
-        :param headers: dict of headers to be appended.
-        :type headers: dict.
+        :param path: The URI to access
+        :type path: str
+        :param args: The query parameters to provide with the request
+        :type args: dict, optional
+        :param body: The request body to provide; use a dict for a JSON body, list for multipart forms, bytes for an octet stream, or str for an unstructured request
+        :type body: dict or list or bytes or str, optional
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int, optional
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int, optional
         :returns: returns a rest request with method 'Put'
 
         """
         return self._rest_request(path, method='PUT', args=args, body=body,
-                                                                headers=headers)
+                                                                headers=headers, timeout=timeout, max_retry=max_retry)
 
-    def patch(self, path, args=None, body=None, headers=None):
-        """Perform a PUT request
+    def patch(self, path, args=None, body=None, headers=None, timeout=None, max_retry=None):
+        """Perform a PATCH request
 
-        :param path: the URL path.
-        :type path: str.
-        :param args: the arguments to patch.
-        :type args: dict.
-        :param body: the body to the sent.
-        :type body: str.
-        :param headers: dict of headers to be appended.
-        :type headers: dict.
+        :param path: The URI to access
+        :type path: str
+        :param args: The query parameters to provide with the request
+        :type args: dict, optional
+        :param body: The request body to provide; use a dict for a JSON body, list for multipart forms, bytes for an octet stream, or str for an unstructured request
+        :type body: dict or list or bytes or str, optional
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int, optional
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int, optional
         :returns: returns a rest request with method 'Patch'
 
         """
         return self._rest_request(path, method='PATCH', args=args, body=body,
-                                                                headers=headers)
+                                                                headers=headers, timeout=timeout, max_retry=max_retry)
 
-    def delete(self, path, args=None, headers=None):
+    def delete(self, path, args=None, headers=None, timeout=None, max_retry=None):
         """Perform a DELETE request
 
-        :param path: the URL path.
-        :type path: str.
-        :param args: the arguments to delete.
-        :type args: dict.
-        :param headers: dict of headers to be appended.
-        :type headers: dict.
+        :param path: The URI to access
+        :type path: str
+        :param args: The query parameters to provide with the request
+        :type args: dict, optional
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int, optional
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int, optional
         :returns: returns a rest request with method 'Delete'
 
         """
         return self._rest_request(path, method='DELETE', args=args,
-                                                                headers=headers)
+                                                                headers=headers, timeout=timeout, max_retry=max_retry)
 
     def _get_req_headers(self, headers=None):
         """Get the request headers
@@ -688,24 +784,34 @@ class RestClientBase(object):
         return headers
 
     def _rest_request(self, path, method='GET', args=None, body=None,
-                      headers=None, allow_redirects=True):
+                      headers=None, allow_redirects=True, timeout=None, max_retry=None):
         """Rest request main function
 
-        :param path: path within tree
+        :param path: The URI to access
         :type path: str
-        :param method: method to be implemented
-        :type method: str
-        :param args: the arguments for method
-        :type args: dict
-        :param body: body payload for the rest call
-        :type body: dict
-        :param headers: provide additional headers
-        :type headers: dict
-        :param allow_redirects: controls whether redirects are followed
-        :type allow_redirects: bool
+        :param method: The HTTP method to invoke on the URI; GET if not provided
+        :type method: str, optional
+        :param args: The query parameters to provide with the request
+        :type args: dict, optional
+        :param body: The request body to provide; use a dict for a JSON body, list for multipart forms, bytes for an octet stream, or str for an unstructured request
+        :type body: dict or list or bytes or str, optional
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
+        :param allow_redirects: Controls whether redirects are followed
+        :type allow_redirects: bool, optional
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int, optional
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int, optional
         :returns: returns a RestResponse object
 
         """
+        if timeout is None:
+            timeout = self._timeout
+
+        if max_retry is None:
+            max_retry = self._max_retry
+
         headers = self._get_req_headers(headers)
         reqpath = path.replace('//', '/')
 
@@ -744,7 +850,7 @@ class RestClientBase(object):
                 body = urlencode(body)
 
             if method == 'PUT':
-                resp = self._rest_request(path=path)
+                resp = self._rest_request(path=path, timeout=timeout, max_retry=max_retry)
 
                 try:
                     if resp.getheader('content-encoding') == 'gzip':
@@ -765,6 +871,7 @@ class RestClientBase(object):
                     LOGGER.error('Error occur while compressing body: %s', excp)
                     raise
 
+        query_str = None
         if args:
             if method == 'GET':
                 # Workaround for this: https://github.com/psf/requests/issues/993
@@ -776,12 +883,12 @@ class RestClientBase(object):
                         none_list.append(query)
                     else:
                         args_copy[query] = args[query]
-                reqpath += '?' + urlencode(args_copy, quote_via=quote)
+                query_str = urlencode(args_copy, quote_via=quote, safe="/?:!$'()*+,;\\=")
                 for query in none_list:
-                    if reqpath[-1] == '?':
-                        reqpath += query
+                    if len(query_str) == 0:
+                        query_str += query
                     else:
-                        reqpath += '&' + query
+                        query_str += '&' + query
             elif method == 'PUT' or method == 'POST' or method == 'PATCH':
                 LOGGER.warning('For POST, PUT and PATCH methods, the provided "args" parameter "{}" is ignored.'
                                .format(args))
@@ -793,20 +900,29 @@ class RestClientBase(object):
         attempts = 0
         restresp = None
         cause_exception = None
-        while attempts <= self._max_retry:
+        while attempts <= max_retry:
             if LOGGER.isEnabledFor(logging.DEBUG):
+                headerstr = ''
+                if headers is not None:
+                    for header in headers:
+                        if header.lower() == "authorization":
+                            headerstr += '\t{}: <REDACTED>\n'.format(header)
+                        else:
+                            headerstr += '\t{}: {}\n'.format(header, headers[header])
                 try:
-                    logbody = None
+                    logbody = 'No request body'
                     if restreq.body:
                         if restreq.body[0] == '{':
-                            logbody = restreq.body
+                            # Mask password properties
+                            # NOTE: If the password itself contains a double quote, it will not redact the entire password
+                            logbody = re.sub(r'"Password"\s*:\s*".*?"', '"Password": "<REDACTED>"', restreq.body)
                         else:
                             raise ValueError('Body of message is binary')
-                    LOGGER.debug('HTTP REQUEST: %s\n\tPATH: %s\n\tBODY: %s'% \
-                                    (restreq.method, restreq.path, logbody))
+                    LOGGER.debug('HTTP REQUEST (%s) for %s:\nHeaders:\n%s\nBody: %s\n'% \
+                                 (restreq.method, restreq.path, headerstr, logbody))
                 except:
-                    LOGGER.debug('HTTP REQUEST: %s\n\tPATH: %s\n\tBODY: %s'% \
-                                (restreq.method, restreq.path, 'binary body'))
+                    LOGGER.debug('HTTP REQUEST (%s) for %s:\nHeaders:\n%s\nBody: %s\n'% \
+                                 (restreq.method, restreq.path, headerstr, 'binary body'))
             attempts = attempts + 1
             LOGGER.info('Attempt %s of %s', attempts, path)
 
@@ -821,8 +937,8 @@ class RestClientBase(object):
                 if self.cafile:
                     verify = self.cafile
                 resp = self._session.request(method.upper(), "{}{}".format(self.__base_url, reqpath), data=body,
-                                             headers=headers, timeout=self._timeout, allow_redirects=allow_redirects,
-                                             verify=verify)
+                                             headers=headers, timeout=timeout, allow_redirects=allow_redirects,
+                                             verify=verify, proxies=self._proxies, params=query_str)
 
                 if sys.version_info < (3, 3):
                     endtime = time.clock()
@@ -851,12 +967,16 @@ class RestClientBase(object):
                         headerstr += '\t' + header[0] + ': ' + header[1] + '\n'
 
                     try:
-                        LOGGER.debug('HTTP RESPONSE for %s:\nCode: %s\nHeaders:\n' \
-                                 '%s\nBody Response of %s: %s'%\
+                        try:
+                            restrespstr = json.dumps(json.loads(restresp.read), indent=4)
+                        except:
+                            restrespstr = restresp.read
+                        LOGGER.debug('HTTP RESPONSE for %s:\nCode: %s\n\nHeaders:\n' \
+                                 '%s\nBody Response of %s:\n%s\n'%\
                                  (restresp.request.path,
                                 str(restresp._http_response.status_code)+ ' ' + \
                                 restresp._http_response.reason,
-                                headerstr, restresp.request.path, restresp.read))
+                                headerstr, restresp.request.path, restrespstr))
                     except:
                         LOGGER.debug('HTTP RESPONSE:\nCode:%s', restresp)
                 else:
@@ -866,7 +986,7 @@ class RestClientBase(object):
         else:
             raise RetriesExhaustedError() from cause_exception
 
-    def login(self, username=None, password=None, auth=AuthMethod.SESSION):
+    def login(self, username=None, password=None, auth=AuthMethod.SESSION, headers=None):
         """Login and start a REST session.  Remember to call logout() when"""
         """ you are done.
 
@@ -876,32 +996,35 @@ class RestClientBase(object):
         :type password: str.
         :param auth: authentication method
         :type auth: object/instance of class AuthMethod
+        :param headers: Additional HTTP headers to provide in the request
+        :type headers: dict, optional
 
         """
+        if getattr(self, "root_resp", None) is None:
+            self.get_root_object()
 
         self.__username = username if username else self.__username
         self.__password = password if password else self.__password
 
+        headers = headers if headers else {}
+
         if auth == AuthMethod.BASIC:
             auth_key = base64.b64encode(('%s:%s' % (self.__username,
                             self.__password)).encode('utf-8')).decode('utf-8')
             self.__authorization_key = 'Basic %s' % auth_key
 
-            headers = dict()
             headers['Authorization'] = self.__authorization_key
 
             respvalidate = self._rest_request(self.login_url, headers=headers)
 
             if respvalidate.status == 401:
-                #If your REST client has a delay for fail attempts add it here
-                delay = 0
-                raise InvalidCredentialsError(delay)
+                # Invalid credentials supplied
+                raise InvalidCredentialsError('HTTP 401 Unauthorized returned: Invalid credentials supplied')
         elif auth == AuthMethod.SESSION:
             data = dict()
             data['UserName'] = self.__username
             data['Password'] = self.__password
 
-            headers = dict()
             resp = self._rest_request(self.login_url, method="POST",body=data,
                                       headers=headers, allow_redirects=False)
 
@@ -910,13 +1033,25 @@ class RestClientBase(object):
             self.__session_key = resp.session_key
             self.__session_location = resp.session_location
 
+            message_item = search_message(resp, "Base", "PasswordChangeRequired")
+            if not message_item is None:
+                raise RedfishPasswordChangeRequiredError("Password Change Required\n", message_item["MessageArgs"][0])
+
             if not self.__session_key and resp.status not in [200, 201, 202, 204]:
-                #If your REST client has a delay for fail attempts added it here
-                delay = 0
-                raise InvalidCredentialsError(delay)
-            else:
-                self.set_username(None)
-                self.set_password(None)
+                if resp.status == 401:
+                    # Invalid credentials supplied
+                    raise InvalidCredentialsError('HTTP 401 Unauthorized returned: Invalid credentials supplied')
+                else:
+                    # Other type of error during session creation
+                    error_str = resp.text
+                    try:
+                        error_str = resp.dict["error"]["@Message.ExtendedInfo"][0]["Message"]
+                    except:
+                        try:
+                            error_str = resp.dict["error"]["message"]
+                        except:
+                            pass
+                    raise SessionCreationError('HTTP {}: Failed to create the session\n{}'.format(resp.status, error_str))
         else:
             pass
 
@@ -944,7 +1079,7 @@ class HttpClient(RestClientBase):
                                 default_prefix='/redfish/v1/',
                                 sessionkey=None, capath=None,
                                 cafile=None, timeout=None,
-                                max_retry=None):
+                                max_retry=None, proxies=None, check_connectivity=True):
         """Initialize HttpClient
 
         :param base_url: The url of the remote system
@@ -965,24 +1100,30 @@ class HttpClient(RestClientBase):
         :type timeout: int
         :param max_retry: Number of times a request will retry after a timeout
         :type max_retry: int
+        :param proxies: Dictionary containing protocol to proxy URL mappings
+        :type proxies: dict
+        :param check_connectivity: A boolean to determine whether the client immediately checks for
+        connectivity to the base_url or not.
+        :type check_connectivity: bool
 
         """
         super(HttpClient, self).__init__(base_url, username=username,
                             password=password, default_prefix=default_prefix,
                             sessionkey=sessionkey, capath=capath,
                             cafile=cafile, timeout=timeout,
-                            max_retry=max_retry)
+                            max_retry=max_retry, proxies=proxies,
+                            check_connectivity=check_connectivity)
 
         try:
             self.login_url = self.root.Links.Sessions['@odata.id']
-        except KeyError:
+        except (KeyError, AttributeError):
             # While the "Links/Sessions" property is required, we can fallback
             # on the URI hardened in 1.6.0 of the specification if not found
             LOGGER.debug('"Links/Sessions" not found in Service Root.')
             self.login_url = '/redfish/v1/SessionService/Sessions'
 
     def _rest_request(self, path='', method="GET", args=None, body=None,
-                      headers=None, allow_redirects=True):
+                      headers=None, allow_redirects=True, timeout=None, max_retry=None):
         """Rest request for HTTP client
 
         :param path: path within tree
@@ -997,18 +1138,17 @@ class HttpClient(RestClientBase):
         :type headers: dict
         :param allow_redirects: controls whether redirects are followed
         :type allow_redirects: bool
+        :param timeout: Timeout in seconds for the initial connection for this specific request
+        :type timeout: int
+        :param max_retry: Number of times a request will retry after a timeout for this specific request
+        :type max_retry: int
         :returns: returns a rest request
 
         """
-        if self.default_prefix == path and path[-1] != '/':
-            path = path + '/'
-        else:
-            pass
-
         return super(HttpClient, self)._rest_request(path=path, method=method,
                                                      args=args, body=body,
                                                      headers=headers,
-                                                     allow_redirects=allow_redirects)
+                                                     allow_redirects=allow_redirects, timeout=timeout, max_retry=max_retry)
 
     def _get_req_headers(self, headers=None, providerheader=None):
         """Get the request headers for HTTP client
@@ -1029,7 +1169,7 @@ def redfish_client(base_url=None, username=None, password=None,
                                 default_prefix='/redfish/v1/',
                                 sessionkey=None, capath=None,
                                 cafile=None, timeout=None,
-                                max_retry=None):
+                                max_retry=None, proxies=None, check_connectivity=True):
     """Create and return appropriate REDFISH client instance."""
     """ Instantiates appropriate Redfish object based on existing"""
     """ configuration. Use this to retrieve a pre-configured Redfish object
@@ -1052,6 +1192,11 @@ def redfish_client(base_url=None, username=None, password=None,
     :type timeout: int
     :param max_retry: Number of times a request will retry after a timeout
     :type max_retry: int
+    :param proxies: Dictionary containing protocol to proxy URL mappings
+    :type proxies: dict
+    :param check_connectivity: A boolean to determine whether the client immediately checks for
+    connectivity to the base_url or not.
+    :type check_connectivity: bool
     :returns: a client object.
 
     """
@@ -1061,4 +1206,4 @@ def redfish_client(base_url=None, username=None, password=None,
     return HttpClient(base_url=base_url, username=username, password=password,
                         default_prefix=default_prefix, sessionkey=sessionkey,
                         capath=capath, cafile=cafile, timeout=timeout,
-                        max_retry=max_retry)
+                        max_retry=max_retry, proxies=proxies, check_connectivity=check_connectivity)
